@@ -9,7 +9,7 @@
 #ifndef PROGRESS_MONITOR_H
 #define PROGRESS_MONITOR_H
 
-#include "global.h"
+#include "common/common/global.h"
 
 class ProgressMonitor : public QObject
 {
