@@ -8,30 +8,9 @@
  *   (at your option) any later version.                                   *
  ***************************************************************************/
 #include "toplevel.h"
+#include "toplevel.moc"
 
-#include <qpixmap.h>
-#include <qiconset.h>
-#include <qlayout.h>
-#include <qsplitter.h>
-#include <qstringlist.h>
-#include <qtimer.h>
-#include <qprogressbar.h>
 #include <qeventloop.h>
-#include <qapplication.h>
-#include <qtooltip.h>
-
-#include <kstatusbar.h>
-#include <klocale.h>
-#include <kedittoolbar.h>
-#include <kapplication.h>
-#include <kconfig.h>
-#include <kaction.h>
-#include <kcmdlineargs.h>
-#include <kio/observer.h>
-#include <kiconloader.h>
-#include <kpopupmenu.h>
-#include <khelpmenu.h>
-#include <kmenubar.h>
 
 #include "gui_debug_manager.h"
 #include "hex_editor.h"
@@ -62,12 +41,15 @@
 #include "main_global.h"
 #include "gui_prog_manager.h"
 #include "devices/gui/register_view.h"
-#include "likeback.h"
-#include "console.h"
 #include "devices/base/device_group.h"
 #include "tools/gui/toolchain_config_center.h"
 #include "global_config.h"
 
+#ifndef NO_KDE
+#  include "likeback.h"
+#  include "console.h"
+#endif
+
 //----------------------------------------------------------------------------
 const ExternalToolType::Data ExternalToolType::DATA[Nb_Types] = {
   { "pikloops", I18N_NOOP("&Pikloops..."),   NULL,   "http://pikloops.sourceforge.net" },
@@ -90,7 +72,7 @@ MainWindow::MainWindow()
 // status bar
     _actionStatus = new QLabel(statusBar());
     statusBar()->addWidget(_actionStatus);
-    _actionProgress = new QProgressBar(statusBar());
+    _actionProgress = new Q3ProgressBar(statusBar());
     statusBar()->addWidget(_actionProgress);
     _debugStatus = new QLabel(statusBar());
     statusBar()->addWidget(_debugStatus, 0, true);
@@ -108,15 +90,14 @@ MainWindow::MainWindow()
     setView(_mainDock);
     setMainDockWidget(_mainDock);
 
-    KIconLoader loader;
     KDockWidget *dock = createDock("project_manager_dock_widget", PURL::icon(PURL::Project),
-                                   i18n("Project Manager"), DockPosition(KDockWidget::DockLeft, 20));
+                                   i18n("Project Manager"), DockPos(KDockWidget::DockLeft, 20));
     Main::_projectManager = new ProjectManager::View(dock);
     connect(Main::_projectManager, SIGNAL(guiChanged()), SLOT(updateGUI()));
     dock->setWidget(Main::_projectManager);
 
-    dock = createDock("watch_view_dock_widget", loader.loadIcon("viewmag", KIcon::Small),
-                      i18n("Watch View"), DockPosition("project_manager_dock_widget"));
+    dock = createDock("watch_view_dock_widget", IconLoader::loadIcon("viewmag", IconLoader::Small),
+                      i18n("Watch View"), DockPos("project_manager_dock_widget"));
     Main::_watchView = new Register::WatchView(dock);
     dock->setWidget(Main::_watchView);
 
@@ -126,28 +107,42 @@ MainWindow::MainWindow()
     connect(Main::_editorManager, SIGNAL(modified(const PURL::Url &)), Main::_projectManager, SLOT(modified(const PURL::Url &)));
     connect(Main::_editorManager, SIGNAL(statusChanged(const QString &)), _editorStatus, SLOT(setText(const QString &)));
 
-    dock = createDock("compile_log_dock_widget", loader.loadIcon("piklab_compile", KIcon::Small),
-                      i18n("Compile Log"), DockPosition(KDockWidget::DockBottom, 80));
+    dock = createDock("compile_log_dock_widget", IconLoader::loadIcon("piklab_compile", IconLoader::Small),
+                      i18n("Compile Log"), DockPos(KDockWidget::DockBottom, 80));
     Main::_compileLog = new Compile::LogWidget(dock);
-    Main::_compileLog->setFocusPolicy(NoFocus);
+#ifdef NO_KDE
+    Main::_compileLog->setFocusPolicy(Qt::NoFocus);
+#else
+    Main::_compileLog->setFocusPolicy(QWidget::NoFocus);
+#endif
     dock->setWidget(Main::_compileLog);
 
-    dock = createDock("program_log_dock_widget", loader.loadIcon("piklab_burnchip", KIcon::Small),
-                      i18n("Program Log"), DockPosition("compile_log_dock_widget"));
+    dock = createDock("program_log_dock_widget", IconLoader::loadIcon("piklab_burnchip", IconLoader::Small),
+                      i18n("Program Log"), DockPos("compile_log_dock_widget"));
     _programLog = new Log::Widget(dock, "program_log");
-    _programLog->setFocusPolicy(NoFocus);
+#ifdef NO_KDE
+    _programLog->setFocusPolicy(Qt::NoFocus);
+#else
+    _programLog->setFocusPolicy(QWidget::NoFocus);
+#endif
     dock->setWidget(_programLog);
 
-    dock = createDock("breakpoints_dock_widget", loader.loadIcon("piklab_breakpoint_active", KIcon::Small),
-                      i18n("Breakpoints"), DockPosition("compile_log_dock_widget"));
+    dock = createDock("breakpoints_dock_widget", IconLoader::loadIcon("piklab_breakpoint_active", IconLoader::Small),
+                      i18n("Breakpoints"), DockPos("compile_log_dock_widget"));
     Main::_breakpointsView = new Breakpoint::View(dock);
-    Main::_breakpointsView->setFocusPolicy(NoFocus);
+#ifdef NO_KDE
+    Main::_breakpointsView->setFocusPolicy(Qt::NoFocus);
+#else
+    Main::_breakpointsView->setFocusPolicy(QWidget::NoFocus);
+#endif
     dock->setWidget(Main::_breakpointsView);
 
-    dock = createDock("console_dock_widget", loader.loadIcon("konsole", KIcon::Small),
-                      i18n("Konsole"), DockPosition("compile_log_dock_widget"));
+#ifndef NO_KDE
+    dock = createDock("console_dock_widget", IconLoader::loadIcon("konsole", IconLoader::Small),
+                      i18n("Konsole"), DockPos("compile_log_dock_widget"));
     Main::_consoleView = new ConsoleView(dock);
     dock->setWidget(Main::_consoleView);
+#endif
 
 // managers
     Programmer::manager = new Programmer::GuiManager(this);
@@ -177,7 +172,11 @@ MainWindow::MainWindow()
     KStdAction::open(this, SLOT(openFile()), actionCollection());
     KRecentFilesAction *recent = KStdAction::openRecent(this, SLOT(openRecentFile(const KURL &)), actionCollection());
     recent->setMaxItems(20);
+#ifdef NO_KDE
+    // ### FIXME
+#else
     recent->loadEntries(kapp->config(), "recent-files");
+#endif
     (void)new KAction(i18n("Save All"), 0, 0, Main::_editorManager, SLOT(saveAllFiles()),
                       actionCollection(), "file_save_all");
     KStdAction::close(Main::_editorManager, SLOT(closeCurrentEditor()), actionCollection());
@@ -204,14 +203,13 @@ MainWindow::MainWindow()
                       Debugger::manager, SLOT(toggleEnableBreakpoint()), actionCollection(), "enable_breakpoint");
     (void)new KAction(i18n("Show disassembly location"), 0, 0,
                       Debugger::manager, SLOT(showDisassemblyLocation()), actionCollection(), "show_disassembly_location");
-    KActionMenu *toolViewsMenu = new KActionMenu( i18n("Tool Views"), 0, "view_tool_views");
+    KActionMenu *toolViewsMenu = new KActionMenu(i18n("Tool Views"), 0, "view_tool_views");
     connect(toolViewsMenu->popupMenu(), SIGNAL(aboutToShow()), SLOT(updateToolViewsActions()));
     actionCollection()->insert(toolViewsMenu);
     a = new KAction(i18n("&Reset Layout"), 0, 0, this, SLOT(resetDockLayout()), actionCollection(), "view_reset_layout");
     toolViewsMenu->insert(a);
     toolViewsMenu->popupMenu()->insertSeparator();
-    QValueList<DockData>::iterator it;
-    for(it=_docks.begin(); it!=_docks.end(); ++it) toolViewsMenu->insert((*it).action);
+    for(QValueList<DockData>::iterator it = _docks.begin(); it != _docks.end(); ++it) toolViewsMenu->insert((*it).action);
 
     // project actions
     (void)new KAction(i18n("New Project..."), "piklab_createproject", 0,
@@ -221,7 +219,11 @@ MainWindow::MainWindow()
     recent = new KRecentFilesAction(i18n("Open Recent Project"), 0,
                       this, SLOT(openRecentProject(const KURL &)), actionCollection(), "project_open_recent");
     recent->setMaxItems(20);
+#ifdef NO_KDE
+    // ### FIXME
+#else
     recent->loadEntries(kapp->config(), "recent-projects");
+#endif
     (void)new KAction(i18n("Project Options..."), "configure", 0,
                       this, SLOT(configureProject()), actionCollection(), "project_options");
     (void)new KAction(i18n("Close Project"), "fileclose", 0,
@@ -316,23 +318,26 @@ MainWindow::MainWindow()
     (void)KStdAction::preferences(this, SLOT(configure()), actionCollection());
 
     // help
+#ifndef NO_KDE
     (void)new KAction(i18n("Report Bug..."), "likeback_bug", 0,
                       LikeBack::instance(), SLOT(iFoundABug()), actionCollection(), "help_report_bug_piklab");
+#endif
 
     setupGUI();
     readDockConfig();
 
+#ifndef NO_KDE
     // LikeBack buttons
     menuBar()->insertItem(new QLabel(menuBar())); // #### first widget is put left-most...
-    MenuBarButton *button = new MenuBarButton("likeback_like", menuBar());
+    MenuBarButton *button = new MenuBarButton("piklab_likeback_like", menuBar());
     QToolTip::add(button, i18n("I like..."));
     connect(button, SIGNAL(clicked()), LikeBack::instance(), SLOT(iLike()));
     menuBar()->insertItem(button);
-    button = new MenuBarButton("likeback_dislike", menuBar());
+    button = new MenuBarButton("piklab_likeback_dislike", menuBar());
     QToolTip::add(button, i18n("I do not like..."));
     connect(button, SIGNAL(clicked()), LikeBack::instance(), SLOT(iDoNotLike()));
     menuBar()->insertItem(button);
-    button = new MenuBarButton("likeback_bug", menuBar());
+    button = new MenuBarButton("piklab_likeback_bug", menuBar());
     QToolTip::add(button, i18n("I found a bug..."));
     connect(button, SIGNAL(clicked()), LikeBack::instance(), SLOT(iFoundABug()));
     menuBar()->insertItem(button);
@@ -343,6 +348,7 @@ MainWindow::MainWindow()
     button = new MenuBarButton("help", menuBar());
     connect(button, SIGNAL(clicked()), LikeBack::instance(), SLOT(showWhatsThisMessage()));
     menuBar()->insertItem(button);
+#endif
 
     QTimer::singleShot(0, this, SLOT(initialLoading()));
 }
@@ -351,7 +357,7 @@ MainWindow::~MainWindow()
 {}
 
 KDockWidget *MainWindow::createDock(const QString &name, const QPixmap &icon,
-                                    const QString &title, const DockPosition &position)
+                                    const QString &title, const DockPos &position)
 {
   KDockWidget *dock = createDockWidget(name, icon, 0, title);
   dock->setDockSite(KDockWidget::DockCenter);
@@ -368,26 +374,30 @@ KDockWidget *MainWindow::createDock(const QString &name, const QPixmap &icon,
 
 void MainWindow::readDockConfig()
 {
+#ifdef NO_KDE
+  // ### FIXME
+#else
   KDockMainWindow::readDockConfig(kapp->config(), "dock_config");
 
   // if there is a new dock: it is not displayed by default...
   QMap<QString, QString> entries = kapp->config()->entryMap("dock_config");
-  QValueList<DockData>::iterator it;
-  for(it=_docks.begin(); it!=_docks.end(); ++it) {
-    QMap<QString, QString>::const_iterator eit;
-    for(eit=entries.begin(); eit!=entries.end(); ++eit)
+  for(QValueList<DockData>::iterator it = _docks.begin(); it != _docks.end(); ++it) {
+    QMap<QString, QString>::const_iterator eit = entries.begin();
+    for(; eit != entries.end(); ++eit)
       if ( eit.key().startsWith((*it).dock->name()) ) break;
     if ( eit==entries.end() ) initDockPosition(*it);
   }
 
   // readDockConfig also restore the names/tooltips: ### what if a new version of the application changes these names...
-  for(it=_docks.begin(); it!=_docks.end(); ++it) (*it).dock->setTabPageLabel((*it).title);
+  for(QValueList<DockData>::iterator it = _docks.begin(); it != _docks.end(); ++it)
+    (*it).dock->setTabPageLabel((*it).title);
   QApplication::postEvent(this, new QEvent(QEvent::CaptionChange));
+#endif
 }
 
 void MainWindow::initialLoading()
 {
-  ::BusyCursor bc;
+  BusyCursorStarter bc;
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
   if ( args->count()!=0 ) { // opened urls provided on the command line
     for (int i = 0; i<args->count(); i++) {
@@ -399,8 +409,7 @@ void MainWindow::initialLoading()
   } else { // otherwise reopen last project/files
     Main::_projectManager->openProject(GlobalConfig::openedProject());
     PURL::UrlList files = GlobalConfig::openedFiles();
-    PURL::UrlList::const_iterator it = files.begin();
-    for (; it!=files.end(); ++it) Main::_editorManager->openEditor(*it);
+    for (PURL::UrlList::const_iterator it = files.begin(); it != files.end(); ++it) Main::_editorManager->openEditor(*it);
   }
   updateGUI();
 }
@@ -412,16 +421,24 @@ void MainWindow::openRecentFile(const KURL &kurl)
 
 void MainWindow::configureToolbar()
 {
+#ifdef NO_KDE
+  // ### FIXME
+#else
   saveMainWindowSettings(KGlobal::config(), "MainWindow");
   KEditToolbar dlg(actionCollection());
   connect(&dlg, SIGNAL(newToolbarConfig()), SLOT(applyToolbarSettings()));
   dlg.exec();
+#endif
 }
 
 void MainWindow::applyToolbarSettings()
 {
+#ifdef NO_KDE
+  // ### FIXME
+#else
   createGUI();
   applyMainWindowSettings(KGlobal::config(), "MainWindow");
+#endif
 }
 
 void MainWindow::configure(ConfigCenter::Type showType)
@@ -437,7 +454,7 @@ void MainWindow::configure(ConfigCenter::Type showType)
 void MainWindow::configureToolchains()
 {
   stopOperations();
-  ToolchainsConfigCenter dialog(*Main::toolGroup(), this);
+  ToolchainsConfigCenter dialog(Main::toolGroup(), this);
   dialog.exec();
   Programmer::manager->clear();
   updateGUI();
@@ -477,38 +494,41 @@ void MainWindow::selectTool(const Tool::Group &group)
   else Debugger::manager->update(true);
 }
 
-void MainWindow::setDevice(const QString &device)
+void MainWindow::setDevice(const Device::Data& data)
 {
-  Compile::Config::setDevice(Main::project(), device);
+  Compile::Config::setDevice(Main::project(), data);
   updateGUI();
 }
 
 void MainWindow::showDeviceInfo()
 {
-  DeviceChooser::Dialog d(Main::device(), this);
+  DeviceChooser::Dialog d(Main::deviceData().name(), this);
   if ( d.exec() ) {
-    setDevice(d.device());
-    updateGUI();
+    const Device::Data* data = Device::Lister::instance().data(d.device());
+    if (data != NULL) {
+      setDevice(*data);
+      updateGUI();
+    }
   }
 }
 
 void MainWindow::configGenerator()
 {
-  PURL::FileType ftype = (Main::currentEditor() ? Main::currentEditor()->fileType() : PURL::Nb_FileTypes);
+  PURL::FileType ftype = (Main::currentEditor() != NULL ? Main::currentEditor()->fileType() : PURL::Nb_FileTypes);
   PURL::SourceFamily family = (ftype!=PURL::Nb_FileTypes ? ftype.data().sourceFamily : PURL::SourceFamily(PURL::SourceFamily::Nb_Types));
   PURL::ToolType type = (family!=PURL::SourceFamily::Nb_Types ? family.data().toolType : PURL::ToolType(PURL::ToolType::Assembler));
   ConfigGenerator dialog(this);
-  dialog.set(Main::deviceData(), *Main::toolGroup(), type);
+  dialog.set(&Main::deviceData(), Main::toolGroup(), type);
   dialog.exec();
 }
 
 void MainWindow::templateGenerator()
 {
-  PURL::FileType ftype = (Main::currentEditor() ? Main::currentEditor()->fileType() : PURL::Nb_FileTypes);
+  PURL::FileType ftype = (Main::currentEditor() != NULL ? Main::currentEditor()->fileType() : PURL::Nb_FileTypes);
   PURL::SourceFamily family = (ftype!=PURL::Nb_FileTypes ? ftype.data().sourceFamily : PURL::SourceFamily(PURL::SourceFamily::Nb_Types));
   PURL::ToolType type = (family!=PURL::SourceFamily::Nb_Types ? family.data().toolType : PURL::ToolType(PURL::ToolType::Assembler));
   TemplateGenerator dialog(this);
-  dialog.set(Main::deviceData(), *Main::toolGroup(), type);
+  dialog.set(&Main::deviceData(), Main::toolGroup(), type);
   dialog.exec();
 }
 
@@ -527,11 +547,16 @@ bool MainWindow::queryClose()
     Main::setState(Main::Idle);
     return false;
   }
+
   // save other settings
-  ::BusyCursor bc;
+  BusyCursorStarter bc;
+#ifdef NO_KDE
+  // ### FIXME
+#else
   writeDockConfig(kapp->config(), "dock_config");
   static_cast<KRecentFilesAction *>(Main::action("project_open_recent"))->saveEntries(kapp->config(), "recent-projects");
   static_cast<KRecentFilesAction *>(Main::action("file_open_recent"))->saveEntries(kapp->config(), "recent-files");
+#endif
   GlobalConfig::writeOpenedProject(Main::_projectManager->projectUrl());
   Main::_projectManager->saveProject();
   return true;
@@ -551,7 +576,7 @@ void MainWindow::newHexFile()
   QString s;
   for (uint i=0; true; i++) {
     s = i18n("Hex") + (i==0 ? QString::null : QString::number(i));
-    if ( Main::_editorManager->findEditor(s)==0 ) break;
+    if (Main::_editorManager->findEditor(s) == NULL) break;
   }
   HexEditor *editor = new HexEditor(s, Main::_editorManager);
   editor->memoryRead();
@@ -628,7 +653,7 @@ void MainWindow::updateGUI()
 
   // update build actions
   bool hexProject = ( Main::_projectManager->projectUrl().fileType()==PURL::Hex );
-  bool customTool = (hasProject ? Main::toolGroup()->isCustom() : false);
+  bool customTool = (hasProject ? Main::toolGroup().isCustom() : false);
   Main::action("build_build_project")->setEnabled(!hexProject && idleProject);
   PURL::Url selected = Main::_projectManager->selectedUrl();
   bool isSelectedSource = ( !selected.isEmpty() && selected.data().group==PURL::Source );
@@ -686,32 +711,36 @@ void MainWindow::updateGUI()
     caption += " - ";
     caption += Main::currentEditor()->url().filepath();
   }
+#ifdef NO_KDE
+  setCaption(caption + " - Piklab");
+#else
   setCaption(KApplication::kApplication()->makeStdCaption(caption));
+#endif
 
   emit deviceChanged();
 }
 
 void MainWindow::updateToolViewsActions()
 {
-  QValueList<DockData>::iterator it;
-  for(it=_docks.begin(); it!=_docks.end(); ++it) (*it).action->setChecked((*it).dock->mayBeHide());
+  for(QValueList<DockData>::iterator it = _docks.begin(); it != _docks.end(); ++it)
+    (*it).action->setChecked((*it).dock->mayBeHide());
 }
 
 void MainWindow::initDockPosition(const DockData &ddata)
 {
-  const DockPosition &pos = ddata.position;
+  const DockPos &pos = ddata.position;
   ddata.dock->manualDock(manager()->getDockWidgetFromName(pos.parent), pos.pos, pos.space);
 }
 
 void MainWindow::resetDockLayout()
 {
-  QValueList<DockData>::iterator it;
-  for (it=_docks.begin(); it!=_docks.end(); ++it) initDockPosition(*it);
+  for (QValueList<DockData>::iterator it = _docks.begin(); it != _docks.end(); ++it)
+    initDockPosition(*it);
 }
 
 void MainWindow::toggleToolView(QWidget *widget)
 {
-  static_cast<KDockWidget *>(widget)->changeHideShowState();
+  static_cast<KDockWidget*>(widget)->changeHideShowState();
 }
 
 void MainWindow::runExternalTool()
@@ -755,12 +784,12 @@ void MainWindow::externalToolDone()
 //-----------------------------------------------------------------------------
 void MainWindow::compileFile()
 {
-  Editor *e = Main::currentEditor();
-  if ( e != NULL && e->isModified() ) e->save(); // buffer is systematically saved
+  Editor* editor = Main::currentEditor();
+  if (editor != NULL && editor->isModified()) editor->save(); // buffer is systematically saved
   stopOperations();
   Main::_compileLog->clear();
-  PURL::Url url = (e != NULL ? e->url() : Main::_projectManager->selectedUrl());
-  bool generated = (e != NULL ? Main::_projectManager->isExternalFile(url) : false);
+  PURL::Url url = (editor != NULL ? editor->url() : Main::_projectManager->selectedUrl());
+  bool generated = (editor != NULL ? Main::_projectManager->isExternalFile(url) : false);
   if ( !generated ) Main::_projectManager->removeExternalFiles();
   Compile::Item item;
   item.url = (!generated ? url : Main::_projectManager->projectUrl());
@@ -774,11 +803,10 @@ void MainWindow::buildProject(Programmer::ProgramAction action)
   _actionAfterBuild = action;
   // save modified buffers
   PURL::UrlList files = Main::project()->absoluteFiles();
-  PURL::UrlList::const_iterator it;
-  for (it=files.begin(); it!=files.end(); ++it) {
+  for (PURL::UrlList::const_iterator it = files.begin(); it != files.end(); ++it) {
     // save modified editors
-    Editor *e = Main::_editorManager->findEditor(*it);
-    if ( e && e->isModified() ) e->save();
+    Editor* editor = Main::_editorManager->findEditor(*it);
+    if (editor != NULL && editor->isModified()) editor->save();
   }
   Programmer::ProgramAction tmp = _actionAfterBuild;
   stopOperations();
@@ -845,8 +873,8 @@ void MainWindow::updateFile(const Compile::FileData &fdata)
     else Q_ASSERT(false);
   }
   if ( fdata.actions & Compile::Show ) {
-    Editor *e = Main::_editorManager->openEditor(fdata.url);
-    if (e) e->setReadOnly(true);
+    Editor *editor = Main::_editorManager->openEditor(fdata.url);
+    if (editor != NULL) editor->setReadOnly(true);
   }
 }
 
@@ -865,23 +893,22 @@ void MainWindow::stopBuild()
 
 void MainWindow::disassembleFile()
 {
-  const HexEditor *he = ::qt_cast<const HexEditor*>(Main::currentEditor());
+  const HexEditor *he = ::qobject_cast<const HexEditor*>(Main::currentEditor());
   if (he == NULL) return;
   stopOperations();
   Main::_compileLog->clear();
   makeWidgetDockVisible(Main::_compileLog);
-  const Device::Data *data = Main::deviceData();
-  Editor* e = new DisassemblyEditor(*he, *data, this);
-  if (!e->open(PURL::Url()) ) {
-    delete e;
-    e = NULL;
-  } else Main::editorManager().addEditor(e);
+  const Device::Data& data = Main::deviceData();
+  std::auto_ptr<Editor> editor(new DisassemblyEditor(*he, data, this));
+  if (editor->open(PURL::Url()) ) {
+    Main::editorManager().addEditor(editor.release());
+  }
 }
 
 //-----------------------------------------------------------------------------
 void MainWindow::keyPressEvent(QKeyEvent *e)
 {
-  if ( e->key()==Key_Escape ) stopOperations();
+  if ( e->key()==Qt::Key_Escape ) stopOperations();
 }
 
 bool MainWindow::stopOperations()
@@ -945,24 +972,25 @@ void MainWindow::closeProject()
 //----------------------------------------------------------------------------
 HexEditor *MainWindow::getHexEditor()
 {
-  if ( Main::_projectManager->isModified() || Main::_projectManager->needsRecompile() ) {
-    MessageBox::Result res = MessageBox::Yes;
-    if ( !readConfigEntry(BaseGlobalConfig::AutoRebuildModified).toBool() ) {
-      res = MessageBox::questionYesNoCancel(i18n("The project hex file may not be up-to-date since some project files have been modified."),
-                                            i18n("Recompile First"), i18n("Continue Anyway"));
-      if ( res==MessageBox::Cancel ) return 0;
-    }
-    if ( res==MessageBox::Yes ) {
-      buildProject(Programmer::ProgramAction::ProgramOnly);
-      return 0;
+  if (Main::project() != NULL) {
+    if ( Main::_projectManager->isModified() || Main::_projectManager->needsRecompile() ) {
+      MessageBox::Result res = MessageBox::Yes;
+      if ( !readConfigEntry(BaseGlobalConfig::AutoRebuildModified).toBool() ) {
+        res = MessageBox::questionYesNoCancel(i18n("The project hex file may not be up-to-date since some project files have been modified."),
+                                              i18n("Recompile First"), i18n("Continue Anyway"));
+        if ( res==MessageBox::Cancel ) return NULL;
+      }
+      if ( res==MessageBox::Yes ) {
+        buildProject(Programmer::ProgramAction::ProgramOnly);
+        return NULL;
+      }
     }
   }
-  if ( Main::currentEditor() && Main::currentEditor()->fileType()==PURL::Hex )
-    return static_cast<HexEditor *>(Main::currentEditor());
+  HexEditor* editor = ::qobject_cast<HexEditor*>(Main::currentEditor());
+  if (editor != NULL) return editor;
   PURL::Url purl = Main::_projectManager->projectUrl();
-  HexEditor *editor = static_cast<HexEditor *>(Main::_editorManager->openEditor(purl.toFileType(PURL::Hex)));
-  if ( editor==0 ) return 0;
-  editor->setReadOnly(true);
+  editor = static_cast<HexEditor *>(Main::_editorManager->openEditor(purl.toFileType(PURL::Hex)));
+  if (editor != NULL) editor->setReadOnly(true);
   return editor;
 }
 
@@ -978,8 +1006,8 @@ void MainWindow::blankCheck()
 
 bool MainWindow::program(bool allowRun)
 {
-  HexEditor *editor = getHexEditor();
-  if ( editor==0 ) return false;
+  const HexEditor *editor = getHexEditor();
+  if (editor == NULL) return false;
   if ( Main::programmerGroup().isDebugger() && !Main::_projectManager->contains(editor->url()) ) {
     MessageBox::sorry(i18n("It is not possible to start a debugging session with an hex file not generated with the current project."), Log::Show);
     return false;
@@ -989,8 +1017,8 @@ bool MainWindow::program(bool allowRun)
 
 void MainWindow::verify()
 {
-  HexEditor *editor = getHexEditor();
-  if ( editor==0 ) return;
+  const HexEditor *editor = getHexEditor();
+  if (editor == NULL) return;
   Programmer::manager->verify(*editor->memory(), Device::MemoryRange());
 }
 
@@ -998,23 +1026,23 @@ void MainWindow::read()
 {
   QString s = i18n("Read");
   Editor *e = Main::_editorManager->findEditor(s);
-  if (e) Main::_editorManager->closeEditor(e, true);
-  HexEditor *editor = new HexEditor(s, Main::_editorManager);
+  if (e != NULL) Main::_editorManager->closeEditor(e, true);
+  std::auto_ptr<HexEditor> editor(new HexEditor(s, Main::_editorManager));
   editor->setDevice();
   if ( Programmer::manager->read(*editor->memory(), Device::MemoryRange()) ) {
     editor->memoryRead();
-    Main::_editorManager->addEditor(editor);
-  } else delete editor;
+    Main::_editorManager->addEditor(editor.release());
+  };
 }
 
 void MainWindow::showProgress(bool show)
 {
   if (show) {
-    BusyCursor::start();
+    _busyCursor.reset(new BusyCursorStarter);
     _actionStatus->show();
     _actionProgress->show();
   } else {
-    BusyCursor::stop();
+    _busyCursor.reset(NULL);
     _actionStatus->hide();
     _actionProgress->hide();
   }
@@ -1024,11 +1052,19 @@ void MainWindow::setTotalProgress(uint nb)
 {
   _actionProgress->setTotalSteps(nb);
   _actionProgress->setProgress(0);
+#ifdef NO_KDE
+  QApplication::processEvents(QEventLoop::ExcludeUserInput); // #### DANGER !!!!
+#else
   QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput); // #### DANGER !!!!
+#endif
 }
 
 void MainWindow::setProgress(uint nb)
 {
   _actionProgress->setProgress(nb);
+#ifdef NO_KDE
+  QApplication::processEvents(QEventLoop::ExcludeUserInput); // #### DANGER !!!!
+#else
   QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput); // #### DANGER !!!!
+#endif
 }
