@@ -10,11 +10,6 @@
 #ifndef TOOLCHAIN_CONFIG_CENTER_H
 #define TOOLCHAIN_CONFIG_CENTER_H
 
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qlineedit.h>
-#include <qwidgetstack.h>
-
 #include "tools/gui/tool_config_widget.h"
 #include "common/gui/dialog.h"
 class ToolchainConfigWidget;
